<?php

namespace Drupal\ad_inserter\Plugin\Block;

use Drupal\ad_inserter\Entity\AdInserter;
use Drupal\Core\Block\Attribute\Block;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'Ad Inserter' Block.
 */
#[Block(
  id: "ad_inserter",
  admin_label: new TranslatableMarkup("Ad Inserter")
)]
class AdInserterBlock extends BlockBase implements ContainerFactoryPluginInterface {

  private AdInserter|null $adInserter = NULL;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * @var EntityStorageInterface
   */
  private EntityStorageInterface $adInserterStorage;

  private ModuleHandlerInterface $moduleHandler;

  /**
   * @param array $configuration
   * @param $plugin_id
   * @param $plugin_definition
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entityTypeManager, ModuleHandlerInterface $moduleHandler) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entityTypeManager;
    $this->adInserterStorage = $entityTypeManager->getStorage('ad_inserter');
    $this->moduleHandler = $moduleHandler;
  }

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('module_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'ad_inserter_id' => NULL,
    ];
  }

  public function blockForm($form, FormStateInterface $form_state) {
    $adInserterId = $this->configuration['ad_inserter_id'];

    if ($adInserterId) {
      $this->adInserter = $this->adInserterStorage->load($adInserterId);
    }
    $form['ad_inserter_id'] = [
      '#title' => t('Select ad inserter to display'),
      '#type' => 'entity_autocomplete',
      '#target_type' => 'ad_inserter',
      '#default_value' => $this->adInserter,
    ];

    return $form;
  }

  public function blockSubmit($form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $this->configuration['ad_inserter_id'] = $values['ad_inserter_id'];
  }

  public function build() {
    $adInserterId = $this->configuration['ad_inserter_id'];
    if (!$adInserterId) {
      return NULL;
    }
    /** @var AdInserter $adInserter */
    $this->adInserter = $this->adInserterStorage->load($adInserterId);
    $options = [
      'status' => TRUE,
    ];
    $this->moduleHandler->invokeAll('ad_inserter_alter_status', [$this->adInserter, &$options]);

    if ($options['status'] === FALSE) {
      return NULL;
    }

    $build['#attributes']['class'][] = 'ad-inserter-block';
    // We always hidden ad inserter before js not showing them.
    if ($this->adInserter->getScreen() != 'all') {
      $build['#attributes']['class'][] = 'hidden';
    }
    $build['#attributes']['data-ad-inserter-id'] = $this->adInserter->id();
    $build['#attributes']['data-ad-inserter-screen'] = $this->adInserter->getScreen();
    $build['#attached']['library'][] = 'ad_inserter/loader';
    $build['ad_inserter'] = $this->entityTypeManager->getViewBuilder('ad_inserter')->view($this->adInserter);
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    if ($this->adInserter) {
      return Cache::mergeContexts(parent::getCacheContexts(), $this->adInserter->getCacheContexts());
    }
    return parent::getCacheContexts();
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    if ($this->adInserter) {
      return Cache::mergeTags(parent::getCacheTags(), $this->adInserter->getCacheTags());
    }
    return parent::getCacheTags();
  }

}
